@@ -31,6 +31,7 @@ static AttrCheck CheckAction;
 static AttrCheck CheckScript;
 static AttrCheck CheckName;
 static AttrCheck CheckId;
+static AttrCheck CheckIs;
 static AttrCheck CheckAlign;
 static AttrCheck CheckValign;
 static AttrCheck CheckBool;
@@ -75,6 +76,7 @@ static AttrCheck CheckRDFaPrefix;
 #define CH_IDREFS      NULL
 #define CH_IDREF       NULL
 #define CH_IDDEF       CheckId
+#define CH_ISDEF       CheckIs
 #define CH_NAME        CheckName
 #define CH_TFRAME      NULL
 #define CH_FBORDER     NULL
@@ -144,6 +146,7 @@ static const Attribute attribute_defs [] =
   { TidyAttr_DEFER,                   "defer",                   CH_BOOL      }, /* SCRIPT */
   { TidyAttr_DIR,                     "dir",                     CH_TEXTDIR   }, /* ltr or rtl */
   { TidyAttr_DISABLED,                "disabled",                CH_BOOL      }, /* form fields */
+  { TidyAttr_DOWNLOAD,                "download",                CH_PCDATA    }, /* anchor */
   { TidyAttr_ENCODING,                "encoding",                CH_PCDATA    }, /* <?xml?> */
   { TidyAttr_ENCTYPE,                 "enctype",                 CH_XTYPE     }, /* FORM */
   { TidyAttr_FACE,                    "face",                    CH_PCDATA    }, /* BASEFONT, FONT */
@@ -160,6 +163,7 @@ static const Attribute attribute_defs [] =
   { TidyAttr_HSPACE,                  "hspace",                  CH_NUMBER    }, /* APPLET, IMG, OBJECT */
   { TidyAttr_HTTP_EQUIV,              "http-equiv",              CH_PCDATA    }, /* META */
   { TidyAttr_ID,                      "id",                      CH_IDDEF     }, 
+  { TidyAttr_IS,                      "is",                      CH_ISDEF     },
   { TidyAttr_ISMAP,                   "ismap",                   CH_BOOL      }, /* IMG */
   { TidyAttr_ITEMID,                  "itemid",                  CH_PCDATA    },
   { TidyAttr_ITEMPROP,                "itemprop",                CH_PCDATA    },
@@ -279,6 +283,7 @@ static const Attribute attribute_defs [] =
   { TidyAttr_CONTENTEDITABLE,         "contenteditable",         CH_PCDATA   },
   { TidyAttr_CONTEXTMENU,             "contextmenu",             CH_PCDATA   },
   { TidyAttr_CONTROLS,                "controls",                CH_PCDATA   },
+  { TidyAttr_CROSSORIGIN,             "crossorigin",             CH_PCDATA   },
   { TidyAttr_DEFAULT,                 "default",                 CH_PCDATA   },
   { TidyAttr_DIRNAME,                 "dirname",                 CH_PCDATA   },
   { TidyAttr_DRAGGABLE,               "draggable",               CH_PCDATA   },
@@ -407,24 +412,14 @@ static const Attribute attribute_defs [] =
   { TidyAttr_ARIA_VALUEMIN,           "aria-valuemin",           CH_PCDATA   },
   { TidyAttr_ARIA_VALUENOW,           "aria-valuenow",           CH_PCDATA   },
   { TidyAttr_ARIA_VALUETEXT,          "aria-valuetext",          CH_PCDATA   },
-
   { TidyAttr_X,                       "x",                       CH_PCDATA    }, /* for <svg> */
   { TidyAttr_Y,                       "y",                       CH_PCDATA    }, /* for <svg> */
-#if 0   /* with uppercase chars taken directly from W3C; are these case-insensitive everywhere? */
-  { TidyAttr_VIEWBOX,                 "viewBox",                 VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-  { TidyAttr_PRESERVEASPECTRATIO,     "preserveAspectRatio",     VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-  { TidyAttr_ZOOMANDPAN,              "zoomAndPan",              VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-  { TidyAttr_BASEPROFILE,             "baseProfile",             VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-  { TidyAttr_CONTENTSCRIPTTYPE,       "contentScriptType",       VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-  { TidyAttr_CONTENTSTYLETYPE,        "contentStyleType",        VERS_INLINE_SVG,   CH_PCDATA    }, /* for <svg> */
-#else
   { TidyAttr_VIEWBOX,                  "viewbox",                CH_PCDATA    }, /* for <svg> */
   { TidyAttr_PRESERVEASPECTRATIO,      "preserveaspectratio",    CH_PCDATA    }, /* for <svg> */
   { TidyAttr_ZOOMANDPAN,               "zoomandpan",             CH_PCDATA    }, /* for <svg> */
   { TidyAttr_BASEPROFILE,              "baseprofile",            CH_PCDATA    }, /* for <svg> */
   { TidyAttr_CONTENTSCRIPTTYPE,        "contentscripttype",      CH_PCDATA    }, /* for <svg> */
   { TidyAttr_CONTENTSTYLETYPE,         "contentstyletype",       CH_PCDATA    }, /* for <svg> */
-#endif
   { TidyAttr_DISPLAY,                  "display",                CH_PCDATA   }, /* on MATH tag (html5) */
 
   /* RDFa Attributes */
@@ -437,6 +432,14 @@ static const Attribute attribute_defs [] =
   { TidyAttr_TYPEOF,                   "typeof",                 CH_RDFATERMS  },
   { TidyAttr_VOCAB,                    "vocab",                  CH_URL        },
 
+  { TidyAttr_INTEGRITY,                "integrity",              CH_PCDATA   },
+
+  /* Preload spec: https://www.w3.org/TR/preload/ */
+  { TidyAttr_AS,                       "as",                     CH_PCDATA   },
+
+  /* for xmlns:xlink in <svg> */
+  { TidyAttr_XMLNSXLINK,                "xmlns:xlink",           CH_URL       },
+
   /* this must be the final entry */
   { N_TIDY_ATTRIBS,                    NULL,                     NULL         }
 };
@@ -579,154 +582,6 @@ static ctmbstr GetColorName(ctmbstr code)
     return NULL;
 }
 
-#if 0
-static const struct _colors fancy_colors[] =
-{
-    { "darkgreen",            "#006400" },
-    { "antiquewhite",         "#FAEBD7" },
-    { "aqua",                 "#00FFFF" },
-    { "aquamarine",           "#7FFFD4" },
-    { "azure",                "#F0FFFF" },
-    { "beige",                "#F5F5DC" },
-    { "bisque",               "#FFE4C4" },
-    { "black",                "#000000" },
-    { "blanchedalmond",       "#FFEBCD" },
-    { "blue",                 "#0000FF" },
-    { "blueviolet",           "#8A2BE2" },
-    { "brown",                "#A52A2A" },
-    { "burlywood",            "#DEB887" },
-    { "cadetblue",            "#5F9EA0" },
-    { "chartreuse",           "#7FFF00" },
-    { "chocolate",            "#D2691E" },
-    { "coral",                "#FF7F50" },
-    { "cornflowerblue",       "#6495ED" },
-    { "cornsilk",             "#FFF8DC" },
-    { "crimson",              "#DC143C" },
-    { "cyan",                 "#00FFFF" },
-    { "darkblue",             "#00008B" },
-    { "darkcyan",             "#008B8B" },
-    { "darkgoldenrod",        "#B8860B" },
-    { "darkgray",             "#A9A9A9" },
-    { "darkgreen",            "#006400" },
-    { "darkkhaki",            "#BDB76B" },
-    { "darkmagenta",          "#8B008B" },
-    { "darkolivegreen",       "#556B2F" },
-    { "darkorange",           "#FF8C00" },
-    { "darkorchid",           "#9932CC" },
-    { "darkred",              "#8B0000" },
-    { "darksalmon",           "#E9967A" },
-    { "darkseagreen",         "#8FBC8F" },
-    { "darkslateblue",        "#483D8B" },
-    { "darkslategray",        "#2F4F4F" },
-    { "darkturquoise",        "#00CED1" },
-    { "darkviolet",           "#9400D3" },
-    { "deeppink",             "#FF1493" },
-    { "deepskyblue",          "#00BFFF" },
-    { "dimgray",              "#696969" },
-    { "dodgerblue",           "#1E90FF" },
-    { "firebrick",            "#B22222" },
-    { "floralwhite",          "#FFFAF0" },
-    { "forestgreen",          "#228B22" },
-    { "fuchsia",              "#FF00FF" },
-    { "gainsboro",            "#DCDCDC" },
-    { "ghostwhite",           "#F8F8FF" },
-    { "gold",                 "#FFD700" },
-    { "goldenrod",            "#DAA520" },
-    { "gray",                 "#808080" },
-    { "green",                "#008000" },
-    { "greenyellow",          "#ADFF2F" },
-    { "honeydew",             "#F0FFF0" },
-    { "hotpink",              "#FF69B4" },
-    { "indianred",            "#CD5C5C" },
-    { "indigo",               "#4B0082" },
-    { "ivory",                "#FFFFF0" },
-    { "khaki",                "#F0E68C" },
-    { "lavender",             "#E6E6FA" },
-    { "lavenderblush",        "#FFF0F5" },
-    { "lawngreen",            "#7CFC00" },
-    { "lemonchiffon",         "#FFFACD" },
-    { "lightblue",            "#ADD8E6" },
-    { "lightcoral",           "#F08080" },
-    { "lightcyan",            "#E0FFFF" },
-    { "lightgoldenrodyellow", "#FAFAD2" },
-    { "lightgreen",           "#90EE90" },
-    { "lightgrey",            "#D3D3D3" },
-    { "lightpink",            "#FFB6C1" },
-    { "lightsalmon",          "#FFA07A" },
-    { "lightseagreen",        "#20B2AA" },
-    { "lightskyblue",         "#87CEFA" },
-    { "lightslategray",       "#778899" },
-    { "lightsteelblue",       "#B0C4DE" },
-    { "lightyellow",          "#FFFFE0" },
-    { "lime",                 "#00FF00" },
-    { "limegreen",            "#32CD32" },
-    { "linen",                "#FAF0E6" },
-    { "magenta",              "#FF00FF" },
-    { "maroon",               "#800000" },
-    { "mediumaquamarine",     "#66CDAA" },
-    { "mediumblue",           "#0000CD" },
-    { "mediumorchid",         "#BA55D3" },
-    { "mediumpurple",         "#9370DB" },
-    { "mediumseagreen",       "#3CB371" },
-    { "mediumslateblue",      "#7B68EE" },
-    { "mediumspringgreen",    "#00FA9A" },
-    { "mediumturquoise",      "#48D1CC" },
-    { "mediumvioletred",      "#C71585" },
-    { "midnightblue",         "#191970" },
-    { "mintcream",            "#F5FFFA" },
-    { "mistyrose",            "#FFE4E1" },
-    { "moccasin",             "#FFE4B5" },
-    { "navajowhite",          "#FFDEAD" },
-    { "navy",                 "#000080" },
-    { "oldlace",              "#FDF5E6" },
-    { "olive",                "#808000" },
-    { "olivedrab",            "#6B8E23" },
-    { "orange",               "#FFA500" },
-    { "orangered",            "#FF4500" },
-    { "orchid",               "#DA70D6" },
-    { "palegoldenrod",        "#EEE8AA" },
-    { "palegreen",            "#98FB98" },
-    { "paleturquoise",        "#AFEEEE" },
-    { "palevioletred",        "#DB7093" },
-    { "papayawhip",           "#FFEFD5" },
-    { "peachpuff",            "#FFDAB9" },
-    { "peru",                 "#CD853F" },
-    { "pink",                 "#FFC0CB" },
-    { "plum",                 "#DDA0DD" },
-    { "powderblue",           "#B0E0E6" },
-    { "purple",               "#800080" },
-    { "red",                  "#FF0000" },
-    { "rosybrown",            "#BC8F8F" },
-    { "royalblue",            "#4169E1" },
-    { "saddlebrown",          "#8B4513" },
-    { "salmon",               "#FA8072" },
-    { "sandybrown",           "#F4A460" },
-    { "seagreen",             "#2E8B57" },
-    { "seashell",             "#FFF5EE" },
-    { "sienna",               "#A0522D" },
-    { "silver",               "#C0C0C0" },
-    { "skyblue",              "#87CEEB" },
-    { "slateblue",            "#6A5ACD" },
-    { "slategray",            "#708090" },
-    { "snow",                 "#FFFAFA" },
-    { "springgreen",          "#00FF7F" },
-    { "steelblue",            "#4682B4" },
-    { "tan",                  "#D2B48C" },
-    { "teal",                 "#008080" },
-    { "thistle",              "#D8BFD8" },
-    { "tomato",               "#FF6347" },
-    { "turquoise",            "#40E0D0" },
-    { "violet",               "#EE82EE" },
-    { "wheat",                "#F5DEB3" },
-    { "white",                "#FFFFFF" },
-    { "whitesmoke",           "#F5F5F5" },
-    { "yellow",               "#FFFF00" },
-    { "yellowgreen",          "#9ACD32" },
-    { NULL,                   NULL      }
-};
-#endif
-
-#if ATTRIBUTE_HASH_LOOKUP
 static uint attrsHash(ctmbstr s)
 {
     uint hashval;
@@ -796,33 +651,24 @@ static void attrsEmptyHash( TidyDocImpl* doc, TidyAttribImpl * attribs )
         attribs->hashtab[i] = NULL;
     }
 }
-#endif
 
 static const Attribute* attrsLookup(TidyDocImpl* doc,
                                TidyAttribImpl* ARG_UNUSED(attribs),
                                ctmbstr atnam)
 {
     const Attribute *np;
-#if ATTRIBUTE_HASH_LOOKUP
     const AttrHash *p;
-#endif
 
     if (!atnam)
         return NULL;
 
-#if ATTRIBUTE_HASH_LOOKUP
     for (p = attribs->hashtab[attrsHash(atnam)]; p && p->attr; p = p->next)
-        if (TY_(tmbstrcmp)(atnam, p->attr->name) == 0)
+        if (TY_(tmbstrcasecmp)(atnam, p->attr->name) == 0)
             return p->attr;
 
     for (np = attribute_defs; np && np->name; ++np)
-        if (TY_(tmbstrcmp)(atnam, np->name) == 0)
+        if (TY_(tmbstrcasecmp)(atnam, np->name) == 0)
             return attrsInstall(doc, attribs, np);
-#else
-    for (np = attribute_defs; np && np->name; ++np)
-        if (TY_(tmbstrcmp)(atnam, np->name) == 0)
-            return np;
-#endif
 
     return NULL;
 }
@@ -917,6 +763,78 @@ AttVal* TY_(RepairAttrValue)(TidyDocImpl* doc, Node* node, ctmbstr name, ctmbstr
         return TY_(AddAttribute)(doc, node, name, value);
 }
 
+
+void TY_(FreeAttrPriorityList)( TidyDocImpl* doc )
+{
+    PriorityAttribs *priorities = &(doc->attribs.priorityAttribs);
+
+    if ( priorities->list )
+    {
+        uint i = 0;
+        while ( priorities->list[i] != NULL )
+        {
+            TidyFree( doc->allocator, priorities->list[i] );
+            i++;
+        }
+
+        TidyFree( doc->allocator, priorities->list );
+    }
+}
+
+
+void TY_(DefinePriorityAttribute)(TidyDocImpl* doc, ctmbstr name)
+{
+    enum { capacity = 10 };
+    PriorityAttribs *priorities = &(doc->attribs.priorityAttribs);
+
+    if ( !priorities->list )
+    {
+        priorities->list = TidyAlloc(doc->allocator, sizeof(ctmbstr) * capacity );
+        priorities->list[0] = NULL;
+        priorities->capacity = capacity;
+        priorities->count = 0;
+    }
+
+    if ( priorities->count >= priorities->capacity )
+    {
+        priorities->capacity = priorities->capacity * 2;
+        priorities->list = realloc( priorities->list, sizeof(tmbstr) * priorities->capacity + 1 );
+    }
+
+    priorities->list[priorities->count] = TY_(tmbstrdup)( doc->allocator, name);
+    priorities->count++;
+    priorities->list[priorities->count] = NULL;
+}
+
+
+TidyIterator TY_(getPriorityAttrList)( TidyDocImpl* doc )
+{
+    PriorityAttribs *priorities = &(doc->attribs.priorityAttribs);
+    size_t result = priorities->count > 0 ? 1 : 0;
+
+    return (TidyIterator) result;
+}
+
+
+ctmbstr  TY_(getNextPriorityAttr)( TidyDocImpl* doc, TidyIterator* iter )
+{
+    PriorityAttribs *priorities = &(doc->attribs.priorityAttribs);
+    size_t index;
+    ctmbstr result = NULL;
+    assert( iter != NULL );
+    index = (size_t)*iter;
+
+    if ( index > 0 && index <= priorities->count )
+    {
+        result = priorities->list[index-1];
+        index++;
+    }
+    *iter = (TidyIterator) ( index <= priorities->count ? index : (size_t)0 );
+
+    return result;
+}
+
+
 static Bool CheckAttrType( TidyDocImpl* doc,
                            ctmbstr attrname, AttrCheck type )
 {
@@ -1192,9 +1110,7 @@ static void FreeDeclaredAttributes( TidyDocImpl* doc )
     while ( NULL != (dict = attribs->declared_attr_list) )
     {
         attribs->declared_attr_list = dict->next;
-#if ATTRIBUTE_HASH_LOOKUP
         attrsRemoveFromHash( doc, &doc->attribs, dict->name );
-#endif
         TidyDocFree( doc, dict->name );
         TidyDocFree( doc, dict );
     }
@@ -1202,9 +1118,7 @@ static void FreeDeclaredAttributes( TidyDocImpl* doc )
 
 void TY_(FreeAttrTable)( TidyDocImpl* doc )
 {
-#if ATTRIBUTE_HASH_LOOKUP
     attrsEmptyHash( doc, &doc->attribs );
-#endif
     TY_(FreeAnchors)( doc );
     FreeDeclaredAttributes( doc );
 }
@@ -1462,17 +1376,75 @@ static void CheckLowerCaseAttrValue( TidyDocImpl* doc, Node *node, AttVal *attva
     }
 }
 
-/* methods for checking value of a specific attribute */
+/* Issue #588 - use simple macros only!
+   Seems 'isalnum(c)' is undefined and can
+   cause an assert or a SIGSEGV in some libraries
+   if 'c' is not EOF, or in the range 0 to 0xff,
+   so avoid using it.
+*/
+#define ISUPPER(a) ((a >= 'A') && (a <= 'Z'))
+#define ISLOWER(a) ((a >= 'a') && (a <= 'z'))
+#define ISNUMERIC(a) ((a >= '0') && (a <= '9'))
+#define ISALNUM(a) (ISUPPER(a) || ISLOWER(a) || ISNUMERIC(a))
+
+static Bool IsURLCodePoint( ctmbstr p, uint *increment )
+{
+    uint c;
+    *increment = TY_(GetUTF8)( p, &c ) + 1;
+
+    return ISALNUM( c ) ||
+        c == '%' ||    /* not a valid codepoint, but an escape sequence */
+        c == '#' ||    /* not a valid codepoint, but a delimiter */
+        c == '!' ||
+        c == '$' ||
+        c == '&' ||
+        c == '\'' ||
+        c == '(' ||
+        c == ')' ||
+        c == '*' ||
+        c == '+' ||
+        c == ',' ||
+        c == '-' ||
+        c == '.' ||
+        c == '/' ||
+        c == ':' ||
+        c == ';' ||
+        c == '=' ||
+        c == '?' ||
+        c == '@' ||
+        c == '_' ||
+        c == '~' ||
+        (c >= 0x00A0 && c <= 0xD7FF) ||
+        (c >= 0xE000 && c <= 0xFDCF) ||
+        (c >= 0xFDF0 && c <= 0xFFEF) ||
+        (c >= 0x10000 && c <= 0x1FFFD) ||
+        (c >= 0x20000 && c <= 0x2FFFD) ||
+        (c >= 0x30000 && c <= 0x3FFFD) ||
+        (c >= 0x40000 && c <= 0x4FFFD) ||
+        (c >= 0x50000 && c <= 0x5FFFD) ||
+        (c >= 0x60000 && c <= 0x6FFFD) ||
+        (c >= 0x70000 && c <= 0x7FFFD) ||
+        (c >= 0x80000 && c <= 0x8FFFD) ||
+        (c >= 0x90000 && c <= 0x9FFFD) ||
+        (c >= 0xA0000 && c <= 0xAFFFD) ||
+        (c >= 0xB0000 && c <= 0xBFFFD) ||
+        (c >= 0xC0000 && c <= 0xCFFFD) ||
+        (c >= 0xD0000 && c <= 0xDFFFD) ||
+        (c >= 0xE0000 && c <= 0xEFFFD) ||
+        (c >= 0xF0000 && c <= 0xFFFFD) ||
+        (c >= 0x100000 && c <= 0x10FFFD);
+}
 
 void TY_(CheckUrl)( TidyDocImpl* doc, Node *node, AttVal *attval)
 {
-    tmbchar c; 
+    tmbchar c;
     tmbstr dest, p;
-    uint escape_count = 0, backslash_count = 0;
+    uint escape_count = 0, backslash_count = 0, bad_codepoint_count = 0;
     uint i, pos = 0;
     uint len;
+    uint increment;
     Bool isJavascript = no;
-    
+
     if (!AttrHasValue(attval))
     {
         TY_(ReportAttrError)( doc, node, attval, MISSING_ATTR_VALUE);
@@ -1480,7 +1452,7 @@ void TY_(CheckUrl)( TidyDocImpl* doc, Node *node, AttVal *attval)
     }
 
     p = attval->value;
-    
+
     isJavascript =
         TY_(tmbstrncmp)(p,"javascript:",sizeof("javascript:")-1)==0;
 
@@ -1496,6 +1468,14 @@ void TY_(CheckUrl)( TidyDocImpl* doc, Node *node, AttVal *attval)
             ++escape_count;
     }
 
+    while ( *p != 0 )
+    {
+        if ( !IsURLCodePoint( p, &increment ) )
+            ++bad_codepoint_count;
+         p = p + increment;
+    }
+    p = attval->value;
+
     if ( cfgBool(doc, TidyFixUri) && escape_count )
     {
         Bool hadnonspace = no;
@@ -1540,11 +1520,15 @@ void TY_(CheckUrl)( TidyDocImpl* doc, Node *node, AttVal *attval)
     {
         if ( cfgBool(doc, TidyFixUri) )
             TY_(ReportAttrError)( doc, node, attval, ESCAPED_ILLEGAL_URI);
-        else
+        else if ( !(TY_(HTMLVersion)(doc) & VERS_HTML5) )
             TY_(ReportAttrError)( doc, node, attval, ILLEGAL_URI_REFERENCE);
 
         doc->badChars |= BC_INVALID_URI;
     }
+    if ( bad_codepoint_count )
+    {
+        TY_(ReportAttrError)( doc, node, attval, ILLEGAL_URI_CODEPOINT );
+    }
 }
 
 /* RFC 2396, section 4.2 states:
@@ -1709,6 +1693,40 @@ void CheckId( TidyDocImpl* doc, Node *node, AttVal *attval )
         AddAnchor( doc, attval->value, node );
 }
 
+void CheckIs( TidyDocImpl* doc, Node *node, AttVal *attval )
+{
+    const char *ptr;
+    Bool go = yes;
+
+    /* `is` MUST NOT be in an autonomous custom tag */
+    ptr = strchr(node->element, '-');
+    if ( ( ptr && (ptr - node->element > 0) ) )
+    {
+        TY_(ReportAttrError)( doc, node, attval, ATTRIBUTE_IS_NOT_ALLOWED);
+    }
+
+    /* Even if we fail the above test, we'll continue to emit reports because
+       the user should *also* know that his attribute values are wrong, even
+       if they should't be in custom tags anyway. */
+
+    /* `is` MUST have a value */
+    if (!AttrHasValue(attval))
+    {
+        TY_(ReportAttrError)( doc, node, attval, MISSING_ATTR_VALUE);
+        return;
+    }
+
+    /* `is` MUST contain a hyphen and no space. */
+    ptr = strchr(attval->value, '-');
+    go = ( ptr && (ptr - attval->value > 0) );
+    ptr = strchr(attval->value, ' ');
+    go = go & (ptr == NULL);
+    if ( !go )
+    {
+        TY_(ReportAttrError)( doc, node, attval, BAD_ATTRIBUTE_VALUE);
+    }
+}
+
 void CheckBool( TidyDocImpl* doc, Node *node, AttVal *attval)
 {
     if (!AttrHasValue(attval))
@@ -2059,15 +2077,15 @@ void CheckType( TidyDocImpl* doc, Node *node, AttVal *attval)
 }
 
 static
-AttVal *SortAttVal( AttVal* list, TidyAttrSortStrategy strat );
+AttVal *SortAttVal( TidyDocImpl* doc, AttVal* list, TidyAttrSortStrategy strat );
 
-void TY_(SortAttributes)(Node* node, TidyAttrSortStrategy strat)
+void TY_(SortAttributes)(TidyDocImpl* doc, Node* node, TidyAttrSortStrategy strat)
 {
     while (node)
     {
-        node->attributes = SortAttVal( node->attributes, strat );
+        node->attributes = SortAttVal( doc, node->attributes, strat );
         if (node->content)
-            TY_(SortAttributes)(node->content, strat);
+            TY_(SortAttributes)(doc, node->content, strat);
         node = node->next;
     }
 }
@@ -2104,25 +2122,88 @@ void TY_(SortAttributes)(Node* node, TidyAttrSortStrategy strat)
 * SOFTWARE.
 */
 
-typedef int(*ptAttValComparator)(AttVal *one, AttVal *two);
+typedef int(*ptAttValComparator)(AttVal *one, AttVal *two, ctmbstr *list);
 
-/* Comparison function for TidySortAttrAlpha */
+/* Returns the index of the item in the array, or -1 if not in the array */
 static
-int AlphaComparator(AttVal *one, AttVal *two)
+int indexof( ctmbstr item, ctmbstr *list )
 {
+    if ( list )
+    {
+        uint i = 0;
+        while ( list[i] != NULL ) {
+            if ( TY_(tmbstrcasecmp)(item, list[i]) == 0 )
+                return i;
+            i++;
+        }
+    }
+
+    return -1;
+}
+
+/* Comparison function for TidySortAttrAlpha. Will also consider items in
+   the passed in list as higher-priority, and will group them first.
+ */
+static
+int AlphaComparator(AttVal *one, AttVal *two, ctmbstr *list)
+{
+    int oneIndex = indexof( one->attribute, list );
+    int twoIndex = indexof( two->attribute, list );
+
+    /* If both on the list, the lower index has priority. */
+    if ( oneIndex >= 0 && twoIndex >= 0 )
+        return oneIndex < twoIndex ? -1 : 1;
+
+    /* If A on the list but B not on the list, then A has priority. */
+    if ( oneIndex >= 0 && twoIndex == -1 )
+        return -1;
+
+    /* If A not on the list but B is on the list, then B has priority. */
+    if ( oneIndex == -1 && twoIndex >= 0 )
+        return 1;
+
+    /* Otherwise nothing is on the list, so just compare strings. */
     return TY_(tmbstrcmp)(one->attribute, two->attribute);
 }
 
 
+/* Comparison function for prioritizing list items. It doesn't otherwise
+   sort.
+ */
+static
+int PriorityComparator(AttVal *one, AttVal *two, ctmbstr *list)
+{
+    int oneIndex = indexof( one->attribute, list );
+    int twoIndex = indexof( two->attribute, list );
+
+    /* If both on the list, the lower index has priority. */
+    if ( oneIndex >= 0 && twoIndex >= 0 )
+        return oneIndex < twoIndex ? -1 : 1;
+
+    /* If A on the list but B not on the list, then A has priority. */
+    if ( oneIndex >= 0 && twoIndex == -1 )
+        return -1;
+
+    /* If A not on the list but B is on the list, then B has priority. */
+    if ( oneIndex == -1 && twoIndex >= 0 )
+        return 1;
+
+    /* Otherwise nothing is on the list, so just mark them as the same. */
+    return 0;
+}
+
+
 /* The "factory method" that returns a pointer to the comparator function */
 static
-ptAttValComparator GetAttValComparator(TidyAttrSortStrategy strat)
+ptAttValComparator GetAttValComparator(TidyAttrSortStrategy strat, ctmbstr *list)
 {
     switch (strat)
     {
     case TidySortAttrAlpha:
         return AlphaComparator;
     case TidySortAttrNone:
+        if ( list && list[0] )
+            return PriorityComparator;
         break;
     }
     return 0;
@@ -2130,9 +2211,12 @@ ptAttValComparator GetAttValComparator(TidyAttrSortStrategy strat)
 
 /* The sort routine */
 static
-AttVal *SortAttVal( AttVal *list, TidyAttrSortStrategy strat)
+AttVal *SortAttVal( TidyDocImpl* doc, AttVal *list, TidyAttrSortStrategy strat)
 {
-    ptAttValComparator ptComparator = GetAttValComparator(strat);
+    /* Get the list from the passed-in tidyDoc. */
+    ctmbstr* priorityList = (ctmbstr*)doc->attribs.priorityAttribs.list;
+
+    ptAttValComparator ptComparator = GetAttValComparator(strat, priorityList);
     AttVal *p, *q, *e, *tail;
     int insize, nmerges, psize, qsize, i;
 
@@ -2143,6 +2227,10 @@ AttVal *SortAttVal( AttVal *list, TidyAttrSortStrategy strat)
     if (!list)
         return NULL;
 
+    /* If no comparator, return the list as-is */
+    if (ptComparator == 0)
+        return list;
+
     insize = 1;
 
     while (1) {
@@ -2176,7 +2264,7 @@ AttVal *SortAttVal( AttVal *list, TidyAttrSortStrategy strat)
                 } else if (qsize == 0 || !q) {
                     /* q is empty; e must come from p. */
                     e = p; p = p->next; psize--;
-                } else if (ptComparator(p,q) <= 0) {
+                } else if (ptComparator(p,q, priorityList) <= 0) {
                     /* First element of p is lower (or same);
                     * e must come from p. */
                     e = p; p = p->next; psize--;
