@@ -36,11 +36,6 @@ struct _Anchor
 
 typedef struct _Anchor Anchor;
 
-#if !defined(ATTRIBUTE_HASH_LOOKUP)
-#define ATTRIBUTE_HASH_LOOKUP 1
-#endif
-
-#if ATTRIBUTE_HASH_LOOKUP
 enum
 {
     ATTRIBUTE_HASH_SIZE=178u
@@ -53,13 +48,19 @@ struct _AttrHash
 };
 
 typedef struct _AttrHash AttrHash;
-#endif
 
 enum
 {
     ANCHOR_HASH_SIZE=1021u
 };
 
+/* Keeps a list of attributes that are sorted ahead of the others. */
+typedef struct _priorityAttribs {
+    tmbstr* list;
+    uint count;
+    uint capacity;
+} PriorityAttribs;
+
 struct _TidyAttribImpl
 {
     /* anchor/node lookup */
@@ -68,9 +69,10 @@ struct _TidyAttribImpl
     /* Declared literal attributes */
     Attribute* declared_attr_list;
 
-#if ATTRIBUTE_HASH_LOOKUP
+    /* Prioritized list of attributes to write */
+    PriorityAttribs priorityAttribs;
+
     AttrHash*  hashtab[ATTRIBUTE_HASH_SIZE];
-#endif
 };
 
 typedef struct _TidyAttribImpl TidyAttribImpl;
@@ -93,6 +95,15 @@ AttVal* TY_(AddAttribute)( TidyDocImpl* doc,
 
 AttVal* TY_(RepairAttrValue)(TidyDocImpl* doc, Node* node, ctmbstr name, ctmbstr value);
 
+/* Add an item to the list of priority attributes to write first. */
+void TY_(DefinePriorityAttribute)(TidyDocImpl* doc, ctmbstr name);
+
+/* Start an iterator for priority attributes. */
+TidyIterator TY_(getPriorityAttrList)( TidyDocImpl* doc );
+
+/* Get the next priority attribute. */
+ctmbstr TY_(getNextPriorityAttr)( TidyDocImpl* doc, TidyIterator* iter );
+
 Bool TY_(IsUrl)( TidyDocImpl* doc, ctmbstr attrname );
 
 /* Bool IsBool( TidyDocImpl* doc, ctmbstr attrname ); */
@@ -132,13 +143,15 @@ void TY_(FreeAnchors)( TidyDocImpl* doc );
 void TY_(InitAttrs)( TidyDocImpl* doc );
 void TY_(FreeAttrTable)( TidyDocImpl* doc );
 
+void TY_(FreeAttrPriorityList)( TidyDocImpl* doc );
+
 void TY_(AppendToClassAttr)( TidyDocImpl* doc, AttVal *classattr, ctmbstr classname );
 /*
  the same attribute name can't be used
  more than once in each element
 */
 void TY_(RepairDuplicateAttributes)( TidyDocImpl* doc, Node* node, Bool isXml );
-void TY_(SortAttributes)(Node* node, TidyAttrSortStrategy strat);
+void TY_(SortAttributes)(TidyDocImpl* doc, Node* node, TidyAttrSortStrategy strat);
 
 Bool TY_(IsBoolAttribute)( AttVal* attval );
 Bool TY_(attrIsEvent)( AttVal* attval );
@@ -184,6 +197,7 @@ Bool TY_(AttributeIsMismatched)(Node* node, AttVal* attval, TidyDocImpl* doc);
 #define attrIsBOTTOMMARGIN(av)      AttrIsId( av, TidyAttr_BOTTOMMARGIN  )
 #define attrIsCELLPADDING(av)       AttrIsId( av, TidyAttr_CELLPADDING  )
 #define attrIsCELLSPACING(av)       AttrIsId( av, TidyAttr_CELLSPACING  )
+#define attrIsCHARSET(av)           AttrIsId( av, TidyAttr_CHARSET  )
 #define attrIsCHAR(av)              AttrIsId( av, TidyAttr_CHAR  )
 #define attrIsCHAROFF(av)           AttrIsId( av, TidyAttr_CHAROFF  )
 #define attrIsCHARSET(av)           AttrIsId( av, TidyAttr_CHARSET  )
@@ -385,6 +399,7 @@ Bool TY_(AttributeIsMismatched)(Node* node, AttVal* attval, TidyDocImpl* doc);
 #define attrGetHEIGHT( nod )      TY_(AttrGetById)( nod, TidyAttr_HEIGHT  )
 #define attrGetFOR( nod )         TY_(AttrGetById)( nod, TidyAttr_FOR  )
 #define attrGetSELECTED( nod )    TY_(AttrGetById)( nod, TidyAttr_SELECTED  )
+#define attrGetCHARSET( nod )     TY_(AttrGetById)( nod, TidyAttr_CHARSET  )
 #define attrGetCHECKED( nod )     TY_(AttrGetById)( nod, TidyAttr_CHECKED  )
 #define attrGetLANG( nod )        TY_(AttrGetById)( nod, TidyAttr_LANG  )
 #define attrGetTARGET( nod )      TY_(AttrGetById)( nod, TidyAttr_TARGET  )
