@@ -18,16 +18,11 @@
 #include "utf8.h"
 
 /* *** FOR DEBUG ONLY *** */
-#if !defined(NDEBUG) && defined(_MSC_VER)
 /* #define DEBUG_PPRINT */
 /* #define DEBUG_INDENT */
-#ifdef DEBUG_PPRINT
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_PPRINT)
 extern void dbg_show_node( TidyDocImpl* doc, Node *node, int caller, int indent );
 #endif
-#ifdef DEBUG_INDENT
-#include "sprtf.h"
-#endif
-#endif
 
 /*
   Block-level and unknown elements are printed on
@@ -69,31 +64,8 @@ void TY_(PPrintSpaces)(void)
     indent_char = ' ';
 }
 
-#if SUPPORT_ASIAN_ENCODINGS
 /* #431953 - start RJ Wraplen adjusted for smooth international ride */
 
-#if 0
-uint CWrapLen( TidyDocImpl* doc, uint ind )
-{
-    ctmbstr lang = cfgStr( doc, TidyLanguage );
-    uint wraplen = cfg( doc, TidyWrapLen );
-
-    if ( !TY_(tmbstrcasecmp)(lang, "zh") )
-        /* Chinese characters take two positions on a fixed-width screen */ 
-        /* It would be more accurate to keep a parallel linelen and wraphere
-           incremented by 2 for Chinese characters and 1 otherwise, but this
-           is way simpler.
-        */
-        return (ind + (( wraplen - ind ) / 2)) ; 
-    
-    if ( !TY_(tmbstrcasecmp)(lang, "ja") )
-        /* average Japanese text is 30% kanji */
-        return (ind + ((( wraplen - ind ) * 7) / 10)) ; 
-    
-    return wraplen;
-}
-#endif
-
 typedef enum
 {
   UC00, /* None                       */
@@ -150,36 +122,6 @@ static struct _unicode4cat
   UnicodeCategory category;
 } const unicode4cat[] =
 {
-#if 0
-  { 0x037E, UCPO }, { 0x0387, UCPO }, { 0x055A, UCPO }, { 0x055B, UCPO },
-  { 0x055C, UCPO }, { 0x055D, UCPO }, { 0x055E, UCPO }, { 0x055F, UCPO },
-  { 0x0589, UCPO }, { 0x058A, UCPD }, { 0x05BE, UCPO }, { 0x05C0, UCPO },
-  { 0x05C3, UCPO }, { 0x05F3, UCPO }, { 0x05F4, UCPO }, { 0x060C, UCPO },
-  { 0x060D, UCPO }, { 0x061B, UCPO }, { 0x061F, UCPO }, { 0x066A, UCPO },
-  { 0x066B, UCPO }, { 0x066C, UCPO }, { 0x066D, UCPO }, { 0x06D4, UCPO },
-  { 0x0700, UCPO }, { 0x0701, UCPO }, { 0x0702, UCPO }, { 0x0703, UCPO },
-  { 0x0704, UCPO }, { 0x0705, UCPO }, { 0x0706, UCPO }, { 0x0707, UCPO },
-  { 0x0708, UCPO }, { 0x0709, UCPO }, { 0x070A, UCPO }, { 0x070B, UCPO },
-  { 0x070C, UCPO }, { 0x070D, UCPO }, { 0x0964, UCPO }, { 0x0965, UCPO },
-  { 0x0970, UCPO }, { 0x0DF4, UCPO }, { 0x0E4F, UCPO }, { 0x0E5A, UCPO },
-  { 0x0E5B, UCPO }, { 0x0F04, UCPO }, { 0x0F05, UCPO }, { 0x0F06, UCPO },
-  { 0x0F07, UCPO }, { 0x0F08, UCPO }, { 0x0F09, UCPO }, { 0x0F0A, UCPO },
-  { 0x0F0B, UCPO }, { 0x0F0D, UCPO }, { 0x0F0E, UCPO }, { 0x0F0F, UCPO },
-  { 0x0F10, UCPO }, { 0x0F11, UCPO }, { 0x0F12, UCPO }, { 0x0F3A, UCPS },
-  { 0x0F3B, UCPE }, { 0x0F3C, UCPS }, { 0x0F3D, UCPE }, { 0x0F85, UCPO },
-  { 0x104A, UCPO }, { 0x104B, UCPO }, { 0x104C, UCPO }, { 0x104D, UCPO },
-  { 0x104E, UCPO }, { 0x104F, UCPO }, { 0x10FB, UCPO }, { 0x1361, UCPO },
-  { 0x1362, UCPO }, { 0x1363, UCPO }, { 0x1364, UCPO }, { 0x1365, UCPO },
-  { 0x1366, UCPO }, { 0x1367, UCPO }, { 0x1368, UCPO }, { 0x166D, UCPO },
-  { 0x166E, UCPO }, { 0x1680, UCZS }, { 0x169B, UCPS }, { 0x169C, UCPE },
-  { 0x16EB, UCPO }, { 0x16EC, UCPO }, { 0x16ED, UCPO }, { 0x1735, UCPO },
-  { 0x1736, UCPO }, { 0x17D4, UCPO }, { 0x17D5, UCPO }, { 0x17D6, UCPO },
-  { 0x17D8, UCPO }, { 0x17D9, UCPO }, { 0x17DA, UCPO }, { 0x1800, UCPO },
-  { 0x1801, UCPO }, { 0x1802, UCPO }, { 0x1803, UCPO }, { 0x1804, UCPO },
-  { 0x1805, UCPO }, { 0x1806, UCPD }, { 0x1807, UCPO }, { 0x1808, UCPO },
-  { 0x1809, UCPO }, { 0x180A, UCPO }, { 0x180E, UCZS }, { 0x1944, UCPO },
-  { 0x1945, UCPO }, 
-#endif
   { 0x2000, UCZS }, { 0x2001, UCZS }, { 0x2002, UCZS }, { 0x2003, UCZS },
   { 0x2004, UCZS }, { 0x2005, UCZS }, { 0x2006, UCZS }, { 0x2008, UCZS },
   { 0x2009, UCZS }, { 0x200A, UCZS }, { 0x2010, UCPD }, { 0x2012, UCPD },
@@ -309,7 +251,6 @@ static WrapPoint Big5WrapPoint(tchar c)
     return NoWrapPoint;
 }
 
-#endif /* SUPPORT_ASIAN_ENCODINGS */
 
 static void InitIndent( TidyIndent* ind )
 {
@@ -600,6 +541,30 @@ static void ResetLineAfterWrap( TidyPrintImpl* pprint )
     ResetLine( pprint );
 }
 
+/*\
+ *  Write the 'indent' char to output
+ *  Issue #335 - The GetSpaces() returns the number of spaces to be
+ *  used for the indent. This is fine if ouputting spaces.
+ *  However, if outputting 'tab' chars, then the number of tabs 
+ *  output should euivalent to spaces divided by 'tab-size'
+\*/
+static void WriteIndentChar(TidyDocImpl* doc )
+{
+    TidyPrintImpl* pprint = &doc->pprint;
+    uint i;
+    uint spaces = GetSpaces(pprint);
+    uint tabsize = cfg(doc, TidyTabSize);
+    if (spaces && (indent_char == '\t') && tabsize)
+    {
+        spaces /= tabsize;  // set number of tabs to output
+        if (spaces == 0)    // with a minimum of one
+            spaces = 1;
+    }
+    for (i = 0; i < spaces; i++)
+        TY_(WriteChar)(indent_char, doc->docOut); /* 20150515 - Issue #108 */
+
+}
+
 /* Goes ahead with writing current line up to
 ** previously saved wrap point.  Shifts unwritten
 ** text in output buffer to beginning of next line.
@@ -613,11 +578,7 @@ static void WrapLine( TidyDocImpl* doc )
         return;
 
     if ( WantIndent(doc) )
-    {
-        uint spaces = GetSpaces( pprint );
-        for ( i = 0; i < spaces; ++i )
-            TY_(WriteChar)( indent_char, doc->docOut ); /* 20150515 - Issue #108 */
-    }
+        WriteIndentChar(doc);
 
     for ( i = 0; i < pprint->wraphere; ++i )
         TY_(WriteChar)( pprint->linebuf[i], doc->docOut );
@@ -653,7 +614,7 @@ static Bool CheckWrapIndent( TidyDocImpl* doc, uint indent )
         WrapLine( doc );
         if ( pprint->indent[ 0 ].spaces < 0 )
         {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
             SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
             pprint->indent[ 0 ].spaces = indent;
@@ -670,11 +631,7 @@ static void WrapAttrVal( TidyDocImpl* doc )
 
     /* assert( IsWrapInAttrVal(pprint) ); */
     if ( WantIndent(doc) )
-    {
-        uint spaces = GetSpaces( pprint );
-        for ( i = 0; i < spaces; ++i )
-            TY_(WriteChar)( indent_char, doc->docOut ); /* 20150515 - Issue #108 */
-    }
+        WriteIndentChar(doc);
 
     for ( i = 0; i < pprint->wraphere; ++i )
         TY_(WriteChar)( pprint->linebuf[i], doc->docOut );
@@ -698,11 +655,7 @@ static void PFlushLineImpl( TidyDocImpl* doc )
     CheckWrapLine( doc );
 
     if ( WantIndent(doc) )
-    {
-        uint spaces = GetSpaces( pprint );
-        for ( i = 0; i < spaces; ++i )
-            TY_(WriteChar)( indent_char, doc->docOut ); /* 20150515 - Issue #108 */
-    }
+        WriteIndentChar(doc);
 
     for ( i = 0; i < pprint->linelen; ++i )
         TY_(WriteChar)( pprint->linebuf[i], doc->docOut );
@@ -725,7 +678,7 @@ void TY_(PFlushLine)( TidyDocImpl* doc, uint indent )
 
     if (pprint->indent[ 0 ].spaces != (int)indent )
     {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
         pprint->indent[ 0 ].spaces = indent;
@@ -747,7 +700,7 @@ static void PCondFlushLine( TidyDocImpl* doc, uint indent )
     /* Issue #390 - Whether chars to flush or not, set new indent */
     if ( pprint->indent[ 0 ].spaces != (int)indent )
     {
-#if !defined(NDEBUG) && defined(_MSC_VER)  && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
          pprint->indent[ 0 ].spaces = indent;
@@ -775,7 +728,7 @@ void TY_(PFlushLineSmart)( TidyDocImpl* doc, uint indent )
 
     if ( pprint->indent[ 0 ].spaces != (int)indent )
     {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
         pprint->indent[ 0 ].spaces = indent;
@@ -799,13 +752,12 @@ static void PCondFlushLineSmart( TidyDocImpl* doc, uint indent )
 
     /*\
      *  Issue #390 - Must still deal with fixing indent!
-     *  If TidyHideEndTags or TidyOmitOptionalTags, then
-     *  in certain circumstance no PrintEndTag will be done,
-     *  so linelen will be 0...
+     *  If TidyOmitOptionalTags, then in cerain circumstances no PrintEndTag
+     *  will be done, so linelen will be 0...
     \*/
     if (pprint->indent[ 0 ].spaces != (int)indent)
     {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
         pprint->indent[ 0 ].spaces = indent;
@@ -899,18 +851,14 @@ static void PPrintChar( TidyDocImpl* doc, uint c, uint mode )
         }
     }
 
-#if SUPPORT_ASIAN_ENCODINGS
-
     /* #431953 - start RJ */
     /* Handle encoding-specific issues */
     switch ( outenc )
     {
     case UTF8:
-#if SUPPORT_UTF16_ENCODINGS
     case UTF16:
     case UTF16LE:
     case UTF16BE:
-#endif
         if (!(mode & PREFORMATTED) && cfg(doc, TidyPunctWrap))
         {
             WrapPoint wp = CharacterWrapPoint(c);
@@ -943,22 +891,6 @@ static void PPrintChar( TidyDocImpl* doc, uint c, uint mode )
         AddChar( pprint, c );
         return;
     }
-    /* #431953 - end RJ */
-
-#else /* SUPPORT_ASIAN_ENCODINGS */
-
-    /* otherwise ISO 2022 characters are passed raw */
-    if (
-#ifndef NO_NATIVE_ISO2022_SUPPORT
-        outenc == ISO2022 ||
-#endif
-        outenc == RAW )
-    {
-        AddChar( pprint, c );
-        return;
-    }
-
-#endif /* SUPPORT_ASIAN_ENCODINGS */
 
     /* don't map latin-1 chars to entities */
     if ( outenc == LATIN1 )
@@ -993,14 +925,12 @@ static void PPrintChar( TidyDocImpl* doc, uint c, uint mode )
         return;
     }
 
-#if SUPPORT_UTF16_ENCODINGS
     /* don't map UTF-16 chars to entities */
     if ( outenc == UTF16 || outenc == UTF16LE || outenc == UTF16BE )
     {
         AddChar( pprint, c );
         return;
     }
-#endif
 
     /* use numeric entities only  for XML */
     if ( cfgBool(doc, TidyXmlTags) )
@@ -1099,14 +1029,6 @@ static void PPrintText( TidyDocImpl* doc, uint mode, uint indent,
     }
 }
 
-#if 0
-static void PPrintString( TidyDocImpl* doc, uint indent, ctmbstr str )
-{
-    while ( *str != '\0' )
-        AddChar( &doc->pprint, *str++ );
-}
-#endif /* 0 */
-
 
 static void PPrintAttrValue( TidyDocImpl* doc, uint indent,
                              ctmbstr value, uint delim, Bool wrappable, Bool scriptAttr )
@@ -1249,7 +1171,7 @@ static void PPrintAttribute( TidyDocImpl* doc, uint indent,
     Bool xmlOut    = cfgBool( doc, TidyXmlOut );
     Bool xhtmlOut  = cfgBool( doc, TidyXhtmlOut );
     Bool wrapAttrs = cfgBool( doc, TidyWrapAttVals );
-    Bool ucAttrs   = cfgBool( doc, TidyUpperCaseAttrs );
+    uint ucAttrs   = cfg( doc, TidyUpperCaseAttrs );
     Bool indAttrs  = cfgBool( doc, TidyIndentAttributes );
     uint xtra      = AttrIndent( doc, node, attr );
     Bool first     = AttrNoIndentFirst( /*doc,*/ node, attr );
@@ -1298,22 +1220,13 @@ static void PPrintAttribute( TidyDocImpl* doc, uint indent,
 
         if (c > 0x7F)
             name += TY_(GetUTF8)(name, &c);
-        else if (ucAttrs)
+        else if (ucAttrs == TidyUppercaseYes)
             c = TY_(ToUpper)(c);
 
         AddChar(pprint, c);
         ++name;
     }
 
-/* fix for bug 732038 */
-#if 0
-    /* If not indenting attributes, bump up indent for 
-    ** value after putting out name.
-    */
-    if ( !indAttrs )
-        indent += xtra;
-#endif
-
     CheckWrapIndent( doc, indent );
  
     if ( attr->value == NULL )
@@ -1585,17 +1498,6 @@ static void PPrintEndTag( TidyDocImpl* doc, uint ARG_UNUSED(mode),
     tmbstr s = node->element;
     tchar c;
 
-   /*
-     Netscape ignores SGML standard by not ignoring a
-     line break before </A> or </U> etc. To avoid rendering 
-     this as an underlined space, I disable line wrapping
-     before inline end tags by the #if 0 ... #endif
-   */
-#if 0
-    if ( !(mode & NOWRAP) )
-        SetWrap( doc, indent );
-#endif
-
     AddString( pprint, "</" );
 
     if (s)
@@ -1624,17 +1526,8 @@ static void PPrintComment( TidyDocImpl* doc, uint indent, Node* node )
     SetWrap( doc, indent );
     AddString( pprint, "<!--" );
 
-#if 0
-    SetWrap( doc, indent );
-#endif
-
     PPrintText(doc, COMMENT, 0, node);
 
-#if 0
-    SetWrap( doc, indent );
-    AddString( pprint, "--" );
-#endif
-
     AddString(pprint, "--");
     AddChar( pprint, '>' );
     if ( node->linebreak && node->next )
@@ -1745,8 +1638,8 @@ static void PPrintXmlDecl( TidyDocImpl* doc, uint indent, Node *node )
     saveWrap = WrapOff( doc );
 
     /* no case translation for XML declaration pseudo attributes */
-    ucAttrs = cfgBool(doc, TidyUpperCaseAttrs);
-    TY_(SetOptionBool)(doc, TidyUpperCaseAttrs, no);
+    ucAttrs = cfg(doc, TidyUpperCaseAttrs);
+    TY_(SetOptionInt)(doc, TidyUpperCaseAttrs, no);
 
     AddString( pprint, "<?xml" );
 
@@ -1760,7 +1653,7 @@ static void PPrintXmlDecl( TidyDocImpl* doc, uint indent, Node *node )
       PPrintAttribute( doc, indent, node, att );
 
     /* restore old config value */
-    TY_(SetOptionBool)(doc, TidyUpperCaseAttrs, ucAttrs);
+    TY_(SetOptionInt)(doc, TidyUpperCaseAttrs, ucAttrs);
 
     if ( node->end <= 0 || doc->lexer->lexbuf[node->end - 1] != '?' )
         AddChar( pprint, '?' );
@@ -1777,9 +1670,6 @@ static void PPrintAsp( TidyDocImpl* doc, uint indent, Node *node )
     Bool wrapJste = cfgBool( doc, TidyWrapJste );
     uint saveWrap = WrapOffCond( doc, !wrapAsp || !wrapJste );
 
-#if 0
-    SetWrap( doc, indent );
-#endif
     AddString( pprint, "<%" );
     PPrintText( doc, (wrapAsp ? CDATA : COMMENT), indent, node );
     AddString( pprint, "%>" );
@@ -1810,9 +1700,6 @@ static void PPrintPhp( TidyDocImpl* doc, uint indent, Node *node )
     TidyPrintImpl* pprint = &doc->pprint;
     Bool wrapPhp = cfgBool( doc, TidyWrapPhp );
     uint saveWrap = WrapOffCond( doc, !wrapPhp  );
-#if 0
-    SetWrap( doc, indent );
-#endif
 
     AddString( pprint, "<?" );
     PPrintText( doc, (wrapPhp ? CDATA : COMMENT),
@@ -1847,9 +1734,6 @@ static void PPrintSection( TidyDocImpl* doc, uint indent, Node *node )
     TidyPrintImpl* pprint = &doc->pprint;
     Bool wrapSect = cfgBool( doc, TidyWrapSection );
     uint saveWrap = WrapOffCond( doc, !wrapSect  );
-#if 0
-    SetWrap( doc, indent );
-#endif
 
     AddString( pprint, "<![" );
     PPrintText( doc, (wrapSect ? CDATA : COMMENT),
@@ -1861,29 +1745,6 @@ static void PPrintSection( TidyDocImpl* doc, uint indent, Node *node )
 }
 
 
-#if 0
-/*
-** Print script and style elements. For XHTML, wrap the content as follows:
-**
-**     JavaScript:
-**         //<![CDATA[
-**             content
-**         //]]>
-**     VBScript:
-**         '<![CDATA[
-**             content
-**         ']]>
-**     CSS:
-**         / *<![CDATA[* /     Extra spaces to keep compiler happy
-**             content
-**         / *]]>* /
-**     other:
-**         <![CDATA[
-**             content
-**         ]]>
-*/
-#endif
-
 static ctmbstr CDATA_START           = "<![CDATA[";
 static ctmbstr CDATA_END             = "]]>";
 static ctmbstr JS_COMMENT_START      = "//";
@@ -1946,6 +1807,8 @@ static Bool TY_(nodeIsTextLike)( Node *node )
         return yes;
     if ( node->type == AspTag )
         return yes;
+    if (node->type == PhpTag)
+        return yes; /* Issue #392 */
     /* add other text like nodes... */
     return no;
 }
@@ -2000,7 +1863,11 @@ void PPrintScriptStyle( TidyDocImpl* doc, uint mode, uint indent, Node *node )
 
     PPrintTag( doc, mode, indent, node );
 
-    TY_(PFlushLineSmart)(doc, indent);
+    /* SCRIPT may have no content such as when loading code via its SRC attribute.
+       In this case we don't want to flush the line, preferring to keep the required
+       closing SCRIPT tag on the same line. */
+    if ( node->content != NULL )
+        TY_(PFlushLineSmart)(doc, indent);
 
     if ( xhtmlOut && node->content != NULL )
     {
@@ -2053,7 +1920,9 @@ void PPrintScriptStyle( TidyDocImpl* doc, uint mode, uint indent, Node *node )
             contentIndent = TextEndsWithNewline( doc->lexer, content, CDATA );
     }
 
-    if ( contentIndent < 0 )
+    /* Only flush the line if these was content present so that the closing
+       SCRIPT tag will stay on the same line. */
+    if ( contentIndent < 0 && node->content != NULL )
     {
         PCondFlushLineSmart( doc, indent );
         contentIndent = 0;
@@ -2076,7 +1945,7 @@ void PPrintScriptStyle( TidyDocImpl* doc, uint mode, uint indent, Node *node )
 
     if ( node->content && pprint->indent[ 0 ].spaces != (int)indent )
     {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
         pprint->indent[ 0 ].spaces = indent;
@@ -2184,7 +2053,7 @@ void TY_(PPrintTree)( TidyDocImpl* doc, uint mode, uint indent, Node *node )
         doc->progressCallback( tidyImplToDoc(doc), node->line, node->column, doc->pprint.line + 1 );
     }
 
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_PPRINT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_PPRINT)
     dbg_show_node( doc, node, 4, GetSpaces( &doc->pprint ) );
 #endif
 
@@ -2343,9 +2212,9 @@ void TY_(PPrintTree)( TidyDocImpl* doc, uint mode, uint indent, Node *node )
         else /* other tags */
         {
             Bool indcont  = ( cfgAutoBool(doc, TidyIndentContent) != TidyNoState );
-            Bool indsmart = ( cfgAutoBool(doc, TidyIndentContent) == TidyAutoState );
-            Bool hideend  = cfgBool( doc, TidyHideEndTags ) ||
-              cfgBool( doc, TidyOmitOptionalTags );
+            /* Issue #582 - Seems this is no longer used
+               Bool indsmart = ( cfgAutoBool(doc, TidyIndentContent) == TidyAutoState ); */
+            Bool hideend  = cfgBool( doc, TidyOmitOptionalTags );
             Bool classic  = TidyClassicVS; /* #228 - cfgBool( doc, TidyVertSpace ); */
             uint contentIndent = indent;
 
@@ -2364,9 +2233,11 @@ void TY_(PPrintTree)( TidyDocImpl* doc, uint mode, uint indent, Node *node )
              *  Issue #180 - with the above PCondFlushLine, 
              *  this adds an uneccessary additional line!
              *  Maybe only if 'classic' ie --vertical-space yes 
+             *  Issue #582 - maybe this is no longer needed!
+             *  It adds a 3rd newline if indent: auto...
+             *  if ( indsmart && node->prev != NULL && classic)
+             *   TY_(PFlushLineSmart)( doc, indent );
             \*/
-            if ( indsmart && node->prev != NULL && classic)
-                TY_(PFlushLineSmart)( doc, indent );
 
             /* do not omit elements with attributes */
             if ( !hideend || !TY_(nodeHasCM)(node, CM_OMITST) ||
@@ -2442,7 +2313,7 @@ void TY_(PPrintTree)( TidyDocImpl* doc, uint mode, uint indent, Node *node )
                     TidyPrintImpl* pprint = &doc->pprint;
                     if (pprint->indent[ 0 ].spaces != (int)indent)
                     {
-#if !defined(NDEBUG) && defined(_MSC_VER) && defined(DEBUG_INDENT)
+#if defined(ENABLE_DEBUG_LOG) && defined(DEBUG_INDENT)
                         SPRTF("%s Indent from %d to %d\n", __FUNCTION__, pprint->indent[ 0 ].spaces, indent );
 #endif  
                         pprint->indent[ 0 ].spaces = indent;
